/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.util.ResizeNotifier");

(function() {
	/* summary:
	 *		provides notification when the browser window resizes, or when 
	 *		triggered programmatically.  By default, registered callbacks will 
	 *		only be fired if the width of the document changes.
	 * 
	 *		To programmatically trigger resize notification:
	 *			dojo.publish("jazz.util.ResizeNotifier.resizeEvent");
	 *
	 *		Supports SideNavigationLayout pane resizing notification.
	 *
	 *		PROVISIONAL: This API is considered provisional and may be changed or removed
	 *		in the future with minimal or no warning.
	 * usage notes:
	 *		- access this object via 'static' methods (e.g.
	 *		  jazz.util.ResizeNotifier.register(args))
	 *		- do not instantiate
	 * since:
	 * 		0.7
	 */		
	 
	var _nodes = new Array();
	var _publishEvents = new Array();
	var _events = new Array();
	var _isInitialized = false;
	var _width = 0;
	var _uniqueId = 0;
	
	jazz.util.ResizeNotifier.register = function(args) {
		/*	args: Object with the following attributes
		 * 		context: object; scope object for callback
		 * 		callback: function; called when resize is detected
		 * 		force: boolean, optional, default false. Always notify function, even if browser width is unchanged
		 * 		publishEvents: boolean, optional, default true.  Notify callback on publish events
		 * 		browserEvents: boolean, optional, default true.  Notify callback on browser resize events
		 * 		priority: 0 = highest priority
		 */
			if (!args.callback) return;
			if (typeof args.browserEvents == "undefined") args.browserEvents = true;
			if (typeof args.publishEvents == "undefined") args.publishEvents = true;
			args.id = _getUniqueId();
			if (args.priority != null && _nodes.length > args.priority) {
				_nodes.splice(args.priority,0,args);
			} else {
				_nodes.push(args);	
			}
			// if this was the first node registered
			if (!_isInitialized)
				_init();

			return args.id;
	};
	jazz.util.ResizeNotifier.unregister = function(id) {
		/*
		 * id: String, returned from register function 
		 */
		for(var i = 0; i < _nodes.length; i++) {
			if (_nodes[i].id == id) {
				_nodes.splice(i,1);
				if (_nodes.length == 0) destroy();
				return;
			}
		}
	};
	jazz.util.ResizeNotifier.poke = function() {
		_onResize(null, true);
	};
	function destroy () {
		_nodes = new Array();
		while(_publishEvents.length > 0)
			dojo.unsubscribe(_publishEvents.pop());
		_publishEvents = new Array();
		while(_events.length > 0)
			dojo.disconnect(_events.pop());
		_events = new Array();
		_isInitialized = false;
	};
	function _init () {
		//Left-nav resize event
		_publishEvents.push(dojo.subscribe("jazz.ui.SideNavigationLayout.slideLeftNav.onEnd",_onLeftNavResize));
		
		//Dashboard resize event (called when a scrollbar could be added - must check width to verify legitimacy)
		_publishEvents.push(dojo.subscribe("jazz.util.ResizeNotifier.resizeEvent",_onResize));
		
		//Window resize event
		_events.push(dojo.connect(window,'onresize',_onResize));
		
		_width = document.documentElement.clientWidth;
		_isInitialized = true;
	};
	function _getUniqueId() {
		return _uniqueId++;
	};
	function _onLeftNavResize() {
		//always notify nodes when left-nav changes
		_onResize(null, true);
	};
	function _onResize(event, isLeftNavResize) {
		for (var i = 0; i < _nodes.length; i++) {
			if (isLeftNavResize || _nodes[i].force || _width != document.documentElement.clientWidth) {
				if (_nodes[i] && (!event || _nodes[i].browserEvents) && (event || _nodes[i].publishEvents) && _nodes[i].callback) {
					try {
						setTimeout(dojo.hitch(_nodes[i].context, _nodes[i].callback), 0);
					} catch (err) {
						console.err("Failed to notify client about resize event: " + err);
					}
				}
			}
		}
		_width = document.documentElement.clientWidth;
	};
})();